# +========================================================================+
# || CAT - CipUX Administration Tool                                      ||
# || CAT-web module: CipUX::CAT::Web::Module::SelfPassword                ||
# ||                                                                      ||
# || Copyright (C) 2010 by Christian Kuelker                              ||
# ||                                                                      ||
# || License: GNU General Public License - GNU GPL - version 2            ||
# ||          or (at your opinion) any later version                      ||
# +========================================================================+
#  ID:       $Id$
#  Revision: $Revision$
#  Head URL: $HeadURL$
#  Date:     $Date$
#  Source:   $Source$

package CipUX::CAT::Web::Module::SelfPassword;

use warnings;
use strict;
use Log::Log4perl qw(get_logger :levels);

# You have to use the Module base class to bootstrap the configuration:
use base qw(CipUX::CAT::Web::Module CipUX::CAT::Web::L10N);

{    # Begin of inside out class

    use version; our $VERSION = qv('3.4.1.0');
    use re 'taint';    # Keep data captured by parentheses tainted
    delete @ENV{qw(PATH IFS CDPATH ENV BASH_ENV)};    # Make %ENV safer

    # CONST
    # Name of the module (without .cgi)
    Readonly::Scalar my $NAME => 'selfpassword';

    # Canonical name of the module (with .cgi)
    Readonly::Scalar my $MODULE => 'selfpassword.cgi';

    # Task to be registered for this module
    # rpc_selfpasswd will use the task
    # cipux_task_change_own_password_interactive
    Readonly::Array my @TASK => qw(
        cipux_task_change_own_password_interactive
    );

    # This sub is mandatory if CipUX tasks are used, otherwise optional
    sub register {

        # +------------------------------------------------------------------+
        # | API
        my $self = shift;

        # +------------------------------------------------------------------+
        # | main
        # Set the module canonical name "selfpassword.cgi"
        $self->set_module_name_register(
            { class => __PACKAGE__, name => $MODULE } );

        # Merge and set the configuration
        my $cfg_ar = $self->module_cfg;
        $self->set_module_cfg_register(
            { cfg_ar => $cfg_ar, name => $MODULE } );

        # +------------------------------------------------------------------+
        # | API
        return 1;
    }

    # This sub is mandatory if CipUX tasks are used, otherwise optional
    # It is also important that CUMULATIVE(BASE FIRST) is used.
    sub module_cfg : CUMULATIVE(BASE FIRST) {

        # +------------------------------------------------------------------+
        # | API
        my $self = shift;

        # +------------------------------------------------------------------+
        # | main
        my $module_hr = {};

        $module_hr->{cipuxName}             = $NAME;
        $module_hr->{cipuxTemplateDir}      = $NAME;
        $module_hr->{cipuxIcon}             = $NAME . '.png';
        $module_hr->{cipuxShortDescription} = 'Set once on password.';

        my $desc = <<"END_DESC";
CipUX-CAT-Web-Selfpasswd is a simple, but full functional module for CAT. You can set you own
new password with it, if you know your old password.
END_DESC

        $module_hr->{cipuxDescription} = $desc;
        $module_hr->{cipuxTask}        = \@TASK;

        # +------------------------------------------------------------------+
        # | API
        # Need to be an array reference of a hash due to cumulative character
        # of Class::Std
        return [$module_hr];
    }

    # This mandatory subroutine triggers the module output
    sub module {

        # +------------------------------------------------------------------+
        # | API
        my ( $self, $arg_r ) = @_;

        # CGI
        my $cgi = $arg_r->{cgi};

        # You may use this object to make RPC calls.
        my $rpc_obj
            = ( exists $arg_r->{rpc_obj} )
            ? $arg_r->{rpc_obj}
            : $self->perr('rpc_obj');
        my $lh_obj
            = ( exists $arg_r->{lh_obj} )
            ? $arg_r->{lh_obj}
            : $self->perr('lh_obj');
        my $c_hr
            = ( exists $arg_r->{c_hr} )
            ? $arg_r->{c_hr}
            : $self->perr('c_hr');

        # +------------------------------------------------------------------+
        # | prepare
        my $logger = get_logger(__PACKAGE__);

        # +------------------------------------------------------------------+
        # | main
        $logger->debug("module [$MODULE]");
        my $login = $rpc_obj->get_login();

        my $p1
            = ( $cgi->param('password1') ) ? $cgi->param('password1') : undef;
        my $p2
            = ( $cgi->param('password2') ) ? $cgi->param('password2') : undef;

        if (    defined $p1
            and defined $p2
            and $p1 ne $EMPTY_STRING
            and $p2 ne $EMPTY_STRING
            and $p1 eq $p2
            and $rpc_obj->rpc_selfpasswd( { password => $p1 } ) )
        {
            $logger->debug("Password information changed for $login.\n");
        }
        else {
            $logger->debug('Set password was not sucessful');
        }

        if ( not defined $p1 or $p1 eq $EMPTY_STRING ) {
            $logger->debug('ERROR:  First input is not given!');
        }
        if ( not defined $p2 or $p2 eq $EMPTY_STRING ) {
            $logger->debug('ERROR: Second input is not given!$todo');
        }

        if ( $p1 ne $p2 ) {
            $logger->debug('ERROR: The two inputs are not identical!');
        }

        # A method to search for the right path. Would be nice if finding the
        # right path in life would be as easy as this ...
        my $path = "tpl/$c_hr->{cat_theme}/$NAME";
        $logger->debug("template path [$path]");

        # add a new language lexicon under the domain 'selfpassword', which is
        # a sub directory under I18N:
        # I18N/selfpassword/message.pot
        # I18N/selfpassword/de.po
        $self->import_lexicon( { domain => 'selfpassword' } );

        # +------------------------------------------------------------------+
        # | API
        # The return of a special hash is mandatory
        return {

           # Point to the layout template, please use word layout.html But you
           # are free as a bird
            layout => "$path/layout.html",

            # This is array reference for trigger and steer the output, one
            # anonymous hash reference for each output part.  This layout_ar
            # have 5 ordered parts:
            #     0. begin_html
            #     1. tt2_hr
            #     2. body_ar
            #     3. footer_hr
            #     4. end_html
            # The layout.html have also 4 parts:
            #     0. [% part0 %]
            #     1. [% part1 %]
            #     2. [% part2 %]
            #     3. [% part3 %]
            #     4. [% part4 %]
            # Valid anonymous hash keys (like begin_html, tt2_hr, ...) are
            # defined in CipUX::CAT::Web::Controller They can be use to choose
            # predefined content or template systems.
            layout_ar => [
                { begin_html => 1, },    # part0
                { body_ar => [ $lh_obj->maketext('User Self Password') ] }
                ,                        # part1
                {
                    tt2_hr => {
                        tpl      => "$path/index.html",
                        param_hr => {
                            DATA   => {login=>$login},
                            MODULE => $MODULE,
                            PATH   => $path,
                            lh     => $lh_obj,
                        },
                    }
                },                       # part2
                { footer_hr => { show_index_back => 1, }, },    # part3
                { end_html => 1, },                             # part4
            ],
        };    # End of return API

    }    # End of sub

}    # End of inside out class

1;   # Magic true value required at end of module

__END__

=pod

=head1 NAME

CipUX::CAT::Web::Module::SelfPassword - Lists all users of the system


=head1 VERSION

This document describes CipUX::CAT::Web::Module::SelfPassword version 3.4.1.0


=head1 SYNOPSIS


This CAT Web module can be used just by copying it into

 $PREFIX/CipUX/CAT/Web/Module

The location of $PREFIX depends on your Perl installation.

It is also possible to use this module stand alone (without cat.cgi):

 package selfpassword; # non conform to scripts (might change)
 use warnings;
 use strict;
 use CipUX::CAT::Web::Module::SelfPassword;
 use CipUX::CAT::Web::Controller;
 use version; our $VERSION = qv('3.4.0.0');

 my $cat = CipUX::CAT::Web::Controller->new(
     { module => 'CipUX::CAT::Web::Module::SelfPassword' } );

 $cat->dispatch( { name => __PACKAGE__ } );

You may find this under doc/example/cgi/selfpassword.pl


=head1 DESCRIPTION

This modules lists all users of the system in one long list, if this modules is
installed, registered and enabled as a CipUX CAT plug-in. It has no intention
of doing something else then showing a simple list. Therefore this module can
be seen as a minimum requirement for writing a CipUX CAT plug-in. Advanced
features should be implemented in other modules. However this list can be used
to module and print a summary of users out of a web browser on screen or paper.


=head1 INTERFACE

=head2 module

Use this method to start the list.


=head1 DIAGNOSTICS

None so far.

=head1 CONFIGURATION AND ENVIRONMENT

CipUX::CAT::Web::Module::SelfPassword requires no configuration files or
environment variables. However it merges its configuration from
CipUX::CAT::Web::Module and itself.


=head1 DEPENDENCIES

 Log::Log4perl
 CipUX::CAT::Web::Module


=head1 INCOMPATIBILITIES

None reported.


=head1 BUGS AND LIMITATIONS

No bugs have been reported.


=head1 AUTHOR

Christian Kuelker  C<< <christian.kuelker@cipworx.org> >>


=head1 LICENCE AND COPYRIGHT

Copyright (C) 2008 -  2010, Christian Kuelker
C<< <christian.kuelker@cipworx.org> >>. All rights reserved.

This module is free software; you can redistribute it and/or modify it under
the GNU GPL licence version 2 or any later version.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

=cut
